'============================================================================================
' ? Copyright, 2005-2008 MicroFour, Inc., All rights reserved.
'
' This software and its associated documentation contain valuable trade secrets 
' and proprietary information belonging to MicroFour, Inc.  None of the software
' or its associated documentation may be stored in a retrieval system, copied,
' transmitted, distributed, transcribed or reproduced in any other way or disclosed
' to any third parties without the express written permission of MicroFour, Inc.  
'
' UNDER NO CIRCUMSTANCES SHALL MICROFOUR, INC. BE LIABLE FOR INCIDENTAL, SPECIAL,
' INDIRECT, DIRECT OR CONSEQUENTIAL DAMAGES OR LOSS OF PROFITS, INTERRUPTION OF 
' BUSINESS, OR RELATED EXPENSES WHICH MAY ARISE FROM THE USE OF SOFTWARE OR DOCUMENTATION, 
' INCLUDING BUT NOT LIMITED TO THOSE RESULTING FROM DEFECTS IN SOFTWARE AND/OR DOCUMENTATION,
' OR LOSS OR INACCURACY OF DATA OF ANY KIND.
'--------------------------------------------------------------------------------------------
' ANY USE OF THIS CODE CONSTITUTES ACCEPTANCE OF THE TERMS OF THE COPYRIGHT NOTICE
'--------------------------------------------------------------------------------------------
' DO NOT REMOVE THIS DISCLAIMER
'============================================================================================

Imports MicroFour.StrataFrame.Business
Imports System.Collections.Generic
Imports System.Web.UI

Namespace UI.Web

    Public Class WebBusinessBindingSourceView
        Inherits DataSourceView

#Region " Constructors "

        ''' <summary>
        ''' Initializes a new instance of the WebBusinessBindingSourceView class.
        ''' </summary>
        ''' <param name="owner"></param>
        ''' <param name="name"></param>
        ''' <param name="bo"></param>
        ''' <remarks></remarks>
        Public Sub New(ByVal owner As WebBusinessBindingSource, ByVal name As String, ByVal bo As BusinessLayer)
            MyBase.New(owner, name)
            Me._BusinessObject = bo
            Me._Source = owner
        End Sub

#End Region

#Region " Private Fields "

        Private _BusinessObject As BusinessLayer
        Private _CurrentEnumerator As WBBSEnumerator
        Private _Source As WebBusinessBindingSource

#End Region

#Region " Public Properties "

        Public Overrides ReadOnly Property CanDelete() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides ReadOnly Property CanInsert() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides ReadOnly Property CanPage() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides ReadOnly Property CanRetrieveTotalRowCount() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides ReadOnly Property CanSort() As Boolean
            Get
                Return True
            End Get
        End Property

        Public Overrides ReadOnly Property CanUpdate() As Boolean
            Get
                Return True
            End Get
        End Property

        ''' <summary>
        ''' Returns the current enumerator being used within the WBBS
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property CurrentEnumerator() As WBBSEnumerator
            Get
                Return _CurrentEnumerator
            End Get
        End Property

#End Region

#Region " Private Methods "

        Private Sub SeekToBoRecord(ByVal keys As IDictionary)
            '-- If the keys count is 0, then the key fields need to be set on the grid
            If keys.Count = 0 OrElse keys.Count <> Me._BusinessObject.PrimaryKeyFields.Length _
                Then Throw New InvalidOperationException("The DataKeyFields must be set to the PrimaryKeys of the business object.")

            '-- Seek to the record that needs to be updated
            Dim pkValues As New List(Of Object)(Me._BusinessObject.PrimaryKeyFields.Length)
            For Each pkField As String In Me._BusinessObject.PrimaryKeyFields
                pkValues.Add(keys(pkField))
            Next
            If Not Me._BusinessObject.SeekToPrimaryKey(New PrimaryKeyValue(pkValues.ToArray())) Then _
                Throw New Exception("The edited record could not be found within the business object.")
        End Sub

#End Region

#Region " Protected Methods "

        Protected Overrides Function ExecuteDelete(ByVal keys As System.Collections.IDictionary, ByVal oldValues As System.Collections.IDictionary) As Integer
            '-- Seek to the record to delete
            Me.SeekToBoRecord(keys)

            '-- Delete the record
            Me._BusinessObject.DeleteCurrentRow(True, True)

            '-- Raise the changed
            Me.OnDataSourceViewChanged(EventArgs.Empty)
        End Function

        Protected Overrides Function ExecuteInsert(ByVal values As System.Collections.IDictionary) As Integer
            '-- Save off the sort
            Dim sort As String = Me._BusinessObject.Sort
            Me._BusinessObject.Sort = String.Empty

            '-- Add a new row & set it's values
            Me._BusinessObject.NewRow()

            For Each field As String In values.Keys
                Me._BusinessObject(field) = values(field)
            Next

            '-- Raise the changed event after setting the sort back
            Me._BusinessObject.Sort = sort
            Me.OnDataSourceViewChanged(EventArgs.Empty)
        End Function

        ''' <summary>
        ''' Executes the select on this view by returning the enumerator that can enumerate over the specified
        ''' rows.
        ''' </summary>
        ''' <param name="arguments"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Protected Overrides Function ExecuteSelect(ByVal arguments As DataSourceSelectArguments) As IEnumerable
            arguments.TotalRowCount = Me._BusinessObject.Count
            Me._BusinessObject.Sort = arguments.SortExpression
            _CurrentEnumerator = New WBBSEnumerator(Me._BusinessObject, arguments.StartRowIndex, arguments.MaximumRows)

            Return _CurrentEnumerator
        End Function

        ''' <summary>
        ''' Executes the update on the specified object.
        ''' </summary>
        ''' <param name="keys"></param>
        ''' <param name="values"></param>
        ''' <param name="oldValues"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Protected Overrides Function ExecuteUpdate(ByVal keys As System.Collections.IDictionary, ByVal values As System.Collections.IDictionary, ByVal oldValues As System.Collections.IDictionary) As Integer
            '-- Seek to the record to edit
            Me.SeekToBoRecord(keys)

            '-- Update the record
            For Each field As String In values.Keys
                Me._BusinessObject(field) = values(field)
            Next

            '-- Raise the changed event
            Me.OnDataSourceViewChanged(EventArgs.Empty)
        End Function

#End Region

        Public Overrides Sub [Select](ByVal arguments As System.Web.UI.DataSourceSelectArguments, ByVal callback As System.Web.UI.DataSourceViewSelectCallback)
            MyBase.[Select](arguments, callback)
        End Sub

#Region " Nested Types "

        Public Structure WBBSEnumerator
            Implements IEnumerator
            Implements IEnumerable

#Region " Constructors "

            ''' <summary>
            ''' Initializes a new instance of this Enumerator structure.
            ''' </summary>
            ''' <param name="bo"></param>
            ''' <param name="start"></param>
            ''' <param name="count"></param>
            ''' <remarks></remarks>
            Public Sub New(ByVal bo As BusinessLayer, ByVal start As Integer, ByVal count As Integer)
                Me._BBS = bo.GetEnumerable(True, False)
                Me._BusinessObjectEnumerator = Me._BBS.GetEnumerator()
                Me._CurrentCount = 0
                Me._MaxCount = count
                Me._StartIndex = start
            End Sub

#End Region

#Region " Private Fields "

            Private _BusinessObjectEnumerator As IEnumerator
            Private _CurrentCount As Integer
            Private _MaxCount As Integer
            Private _StartIndex As Integer

            Private _BBS As IEnumerable
#End Region

            ''' <summary>
            ''' Gets a reference to the current object of this enumerator
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            Public ReadOnly Property Current() As Object Implements IEnumerator.Current
                Get
                    Return Me._BusinessObjectEnumerator.Current
                End Get
            End Property

            ''' <summary>
            ''' Moves this enumerator to the next item.
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            Public Function MoveNext() As Boolean Implements IEnumerator.MoveNext
                '-- If the current count is 0, then this is the first operation, and we need to move
                '   the pointer up to the start index
                If Me._CurrentCount = 0 Then
                    For i As Integer = 0 To Me._StartIndex - 1
                        If Not Me._BusinessObjectEnumerator.MoveNext() Then Return False
                    Next
                End If

                '-- If the current count is greater than the max count, then we're done, so return false
                If Me._MaxCount > 0 AndAlso Me._CurrentCount >= Me._MaxCount Then Return False

                '-- Otherwise, just allow the contained enumerator to return
                Me._CurrentCount += 1
                Return Me._BusinessObjectEnumerator.MoveNext()
            End Function

            ''' <summary>
            ''' Resets this enumerator.
            ''' </summary>
            ''' <remarks></remarks>
            Public Sub Reset() Implements IEnumerator.Reset
                '-- Resets this enumerator
                Me._CurrentCount = 0
                Me._BusinessObjectEnumerator.Reset()
            End Sub

            ''' <summary>
            ''' Returns an enumerator that can enumerate over the objects within this view.
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            Public Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
                '-- Return this object
                Return Me
            End Function

            Public ReadOnly Property BOEnumerator() As BusinessLayerEnumerator
                Get
                    Return CType(Me._BusinessObjectEnumerator, BusinessLayerEnumerator)
                End Get
            End Property

            Public ReadOnly Property BBS() As BusinessBindingSource
                Get

                    Return CType(Me._BBS, BusinessBindingSource)
                End Get
            End Property
        End Structure

#End Region

    End Class

End Namespace
